#! /usr/bin/env python

# Genetic
# Copyright (C) 2001 Jean-Baptiste LAMY
#
# This program is free software. See README or LICENSE for the license terms.

"""Genetic demo 4

This demo shows the organism's capability to loose their mutation ability
and stop evoluting !

This often occurs when the "problem" is solved (and it can be a good way
to know WHEN it is solved !), but sometime not...

In this demo, the initial/starting organisms are already perfect; so they will
loose their mutation power. Obviously, because any mutation is BAD for them !
So they will reject the __mutation__ gene as they have rejected the
__deletion__ in the previous demo.
"""

import random, string, math
from genetic import organism, lifecycle


# Set the function to minimize.

FUNC = lambda x, y: x ** 2 + y ** 2

class FUNCMinimizer(organism.Organism):
  def compute_phenotypes(self):
    self.func_value, self.func_args = organism.recessive_phenotype(self.genotype, FUNC)
    if self.func_value is None:
      # This organism is not viable.
      self.canlive = 0
      
  def __cmp__(self, other):
    return cmp(self.func_value, other.func_value)
  
  def __repr__(self):
    return "Minimum: FUNC%s = %s\n" % (self.func_args, self.func_value) + organism.Organism.__repr__(self)


# Create an initial set of organisms.
#
# This time, initial organisms are already "perfect" : they minimize FUNC !
#
# As organisms don't need to evoluate, you will see __mutation__ and
# __mutampl__ falling down ! ANY mutation is bad here !

organismA = FUNCMinimizer([
  organism.Chromosom(x = 0.0, y = -0.0),
  ])

organismB = FUNCMinimizer([
  organism.Chromosom(x = -0.0, y = 0.0),
  ])

organisms = [organismA, organismB]


# Define some parameters

# True for elitism (= always keep the best organism) -- We MUST disable it here ! (Else we'll get the first organism at the end !)
ELITISM = 0

# Number of generation to run
NB_GENERATION = 10

# Number of children each generation made
NB_CHILDREN  = 100

# Number of organisms kept per generation (other children doesn't survive)
NB_ORGANISMS = 10


# Ready !

lifecycle.run(organisms, ELITISM, NB_GENERATION, NB_CHILDREN, NB_ORGANISMS)
