#! /usr/bin/env python

# Genetic
# Copyright (C) 2001 Jean-Baptiste LAMY
#
# This program is free software. See README or LICENSE for the license terms.

"""Genetic demo 5

This demo shows how organisms control the amplitude of their mutation -- the
famous __mutampl__ magic gene.

Here, we minimize the function lambda x: x  (or f(x) = x). This function
minimize in negative infinity !
So the mutation amplitude __mutampl__ will dramatically increase !
"""

import random, string, math
from genetic import organism, lifecycle

# Set the function to minimize. f(x) = x minimize in -infinity !

def FUNC(x): return x

class FUNCMinimizer(organism.Organism):
  def compute_phenotypes(self):
    self.func_value, self.func_args = organism.recessive_phenotype(self.genotype, FUNC)
    if self.func_value is None:
      # This organism is not viable.
      self.canlive = 0
      
  def __cmp__(self, other):
    return cmp(self.func_value, other.func_value)
  
  def __repr__(self):
    return "Minimum: FUNC%s = %s\n" % (self.func_args, self.func_value) + organism.Organism.__repr__(self)


# Create the initial set of organisms.

organismA = FUNCMinimizer([
  organism.Chromosom(x = 5.0, __break__ = 0.0),
  ])

organismB = FUNCMinimizer([
  organism.Chromosom(x = 1.0, __break__ = 0.0),
  ])

organisms = [organismA, organismB]


# Define some parameters
# Here (and its almost always true with Genetic), we have a much better result
# if we use a lot of generations of few organisms -- this given more time
# to __mutampl__ to mutate and increase !

# True for elitism (= always keep the best organism)
ELITISM = 1

# Number of generation to run
NB_GENERATION = 60

# Number of children each generation made
NB_CHILDREN  = 8

# Number of organisms kept per generation (other children doesn't survive)
NB_ORGANISMS = 3


# Ready !

lifecycle.run(organisms, ELITISM, NB_GENERATION, NB_CHILDREN, NB_ORGANISMS)

# This time we see __mutation__ and __mutampl__ increasing quickly !
#
# Obviously the final result IS NOT the minimum, as we'll never obtain infinity. But, at the
# end of the experience, if you can see that __mutation__ and / or __mutampl__ are high,
# this clearly means that you are not arrived at the real minimum, since the organisms are
# not stable !
