#! /usr/bin/env python# Define the function to minimize.
# 
# Here, we'll use a MutableOrderList, i.e. a list whose order can mutate
# (= item are swapped in the list).
# 
# MutableOrderList extends the Mutable class and can be used as a value for a
# gene, instead of a float or anything. Here we call this list "order"

def journey_distance(order):
  # order is : [first_city, second_city, ...]
  return reduce(operator.add, map(lambda cityA, cityB: math.sqrt((cityA.x - cityB.x) ** 2 + (cityA.y - cityB.y) ** 2), order, order[1:] + [order[0]]))


# Genetic
# Copyright (C) 2001 Jean-Baptiste LAMY
#
# This program is free software. See README or LICENSE for the license terms.

"""Genetic circle demo

This demo search the smaller circle that contains all the given points.
"""

from __future__ import nested_scopes

import random, string, math, Tkinter
from genetic import organism, lifecycle

SIZE = 250


class Point:
  def __init__(self, x, y):
    self.x = x
    self.y = y
    
  def __repr__(self): return "<Point %s, %s>" % (self.x, self.y)
  
points = []

def radius_func(x, y):
  # return the largest distance from the center (x, y) to one of the points.
  if len(points) == 0: return 0.0
  return max(map(lambda point: math.sqrt((point.x - x) ** 2 + (point.y - y) ** 2), points))

class CircleMinimizer(organism.Organism):
  def compute_phenotypes(self):
    self.radius, self.radius_args = organism.perchromosom_dominancy_phenotype(self.genotype, radius_func)
    if self.radius is None: # This organism is not viable.
      self.canlive = 0
      
  def __cmp__(self, other):
    # Say how we compare 2 organisms. Here, the lower the circle radius is the better the organism is.
    return cmp(self.radius, other.radius)

  def __repr__(self):
    return "circle radius : %s\n" % self.radius + organism.Organism.__repr__(self)

class Graph(Tkinter.Canvas):
  def __init__(self, master):
    Tkinter.Canvas.__init__(self, master, bg = "white", width = SIZE, height = SIZE)
    self.bind("<Button-1>", self.leftClick )
    self.circle = None
    
    organismA = CircleMinimizer([
      organism.Chromosom(x = 50.0, y = 120.0, __mutampl__ = 50.0, __mutation__ = 0.8, __break__ = 0.0),
      ])
    organismB = CircleMinimizer([
      organism.Chromosom(x = 100.0, y = 100.0, __mutampl__ = 20.0, __mutation__ = 0.8, __break__ = 0.0),
      ])
    self.organisms = [organismA, organismB]
    
  def recomputephenotypes(self):
    for organism in self.organisms: organism.compute_phenotypes()
    
  def leftClick (self, event): self.addPoint(Point(event.x, event.y))
  
  def addPoint(self, point):
    points.append(point)
    self.create_text(point.x - 0, point.y - 0, text = "+")
    # Phenotypes have changed, because there is one point more !
    self.recomputephenotypes()
    
  def generation(self, nb = 1):
    self.organisms = lifecycle.run(self.organisms, 1, nb, 20, 8)
    
  def update(self):
    if self.circle: self.delete(self.circle)
    best   = self.organisms[0]
    radius = best.radius
    x, y   = best.radius_args
    self.circle = self.create_oval(x - radius, y - radius, x + radius, y + radius)
    

class GraphFrame(Tkinter.Tk):
  def __init__(self):
    Tkinter.Tk.__init__(self, className = "genetic circle")
    Tkinter.Label(self, text = "Click some points below, then run 1 or 10 generations !\nI will genetically compute the smallest circle around your points !", wraplength = SIZE + 30).pack()
    self.graph = Graph(self)
    self.graph.pack()
    Tkinter.Button(self, text = "1   generation" , command = self.generation1  ).pack()
    Tkinter.Button(self, text = "10  generations", command = self.generation10 ).pack()
        
  def generation1(self, event = None):
    self.graph.generation(1)
    self.graph.update()
        
  def generation10(self, event = None):
    self.graph.generation(10)
    self.graph.update()
    

graphframe = GraphFrame()
Tkinter.mainloop()
