#! /usr/bin/env python

# Genetic
# Copyright (C) 2001 Jean-Baptiste LAMY
#
# This program is free software. See README or LICENSE for the license terms.

from __future__ import nested_scopes

from genetic import organism, lifecycle, neural
import Tkinter, operator


SIZE = 250.0
STEP =  10.0

class Point:
  def __init__(self, x, y, value):
    self.x     = x
    self.y     = y
    self.value = value
    
  def __repr__(self): return "<Point %s, %s value %s>" % (self.x, self.y, self.value)
  
points = []

def value2color(value):
  if   value < -1.0: value = -1.0
  elif value >  1.0: value =  1.0
  return "gray" + str(int((value + 1.0) * 50.0))


neuronX     = neural.Input("x")
neuronY     = neural.Input("y")
neuronconst = neural.Constant(0.5)


def square(neuralfunc):
  def func(point):
    value = neuralfunc(x = point.x / SIZE, y = point.y / SIZE)
    if   value < -1.0: value = -1.0
    elif value >  1.0: value =  1.0
    return (value - point.value) ** 2
  
  return reduce(operator.add, map(func, points), 0.0)
  #return reduce(operator.add, map(lambda point: (neuralfunc(x = point.x, y = point.y) - point.value) ** 2, points), 0.0)


class PointsSeparator(organism.Organism):
  characteristics = [
    organism.Characteristic("square", square, organism.RECESSIVE_PHENOTYPE)
    ]
  
  def __cmp__(self, other):
    # Say how we compare 2 organisms. Here, the lower the square is, the better the organism is.
    return cmp(self.square, other.square)


class Graph(Tkinter.Canvas):
  def __init__(self, master):
    Tkinter.Canvas.__init__(self, master, bg = "gray50", width = SIZE, height = SIZE)
    self.bind("<Button-1>", self.leftClick )
    self.bind("<Button-3>", self.rightClick)
    self.rects = []
    self.best  = None
    
    nn = neural.Neuron()
    nn.synapses.append(neural.Synaps(neuronX    ,  1.1))
    nn.synapses.append(neural.Synaps(neuronY    , -3.3))
    nn.synapses.append(neural.Synaps(neuronconst,  0.0 ))
    organismA = PointsSeparator([
      nn,
      ])
    
    nn = neural.Neuron()
    nn.synapses.append(neural.Synaps(neuronX    ,  2.2))
    nn.synapses.append(neural.Synaps(neuronY    , -1.1))
    nn.synapses.append(neural.Synaps(neuronconst,  3.3))
    organismB = PointsSeparator([
      nn,
      ])
    
    self.organisms = [organismA, organismB]
    
  def recomputephenotypes(self):
    for organism in self.organisms: organism.compute_phenotypes()
    
  def leftClick  (self, event): self.addPoint(Point(event.x, event.y, -1.0))
  def rightClick (self, event): self.addPoint(Point(event.x, event.y,  1.0))
  
  def addPoint(self, point):
    points.append(point)
    self.create_rectangle(point.x - 3, point.y - 3, point.x + 4, point.y + 4, fill = "gray50", outline = "")
    if point.value > 0.0: self.create_text(point.x, point.y, text = "+", fill = "white")
    else:                 self.create_text(point.x, point.y, text = "+", fill = "black")
    # Phenotypes have changed, because there is one point more !
    self.recomputephenotypes()
    
  def generation(self, nb = 1):
    self.organisms = lifecycle.run(self.organisms, 1, nb, 40, 15)
    
  def update(self):
    best = self.organisms[0]
    if not best is self.best: # If it is still the same, it's useless to update !
      self.best = self
      
      neuralnetwork, = best.square_args
      
      self.delete(self.rects)
      DEMI_STEP = STEP / 2.0
      for i in range(0, SIZE, STEP):
        for j in range(0, SIZE, STEP):
          self.rects.append(self.create_rectangle(i, j, i + STEP, j + STEP, tags = "rect", outline = "", fill = value2color(neuralnetwork(x = (i + STEP / DEMI_STEP) / SIZE, y = (j + DEMI_STEP) / SIZE))))
          
      self.tag_lower("rect")
      
    

class GraphFrame(Tkinter.Tk):
  def __init__(self):
    Tkinter.Tk.__init__(self, className = "genetic separator")
    Tkinter.Label(self, text = "Left-click below to add a black point, and right-click to add a white one, then run 1 or 10 generations !\nI will genetically separe black and white points !", wraplength = SIZE + 30).pack()
    self.graph = Graph(self)
    self.graph.pack()
    Tkinter.Button(self, text = "1   generation" , command = self.generation1  ).pack()
    Tkinter.Button(self, text = "10  generations", command = self.generation10 ).pack()
    
  def generation1(self, event = None):
    self.graph.generation(1)
    self.graph.update()
        
  def generation10(self, event = None):
    self.graph.generation(10)
    self.graph.update()
    

graphframe = GraphFrame()
Tkinter.mainloop()
