# Genetic
# Copyright (C) 2001 Jean-Baptiste LAMY
#
# This program is free software. See README or LICENSE for the license terms.

"""genetic.mutablelist -- module for special mutable list
"""

import operator, random, UserList, copy
from genetic import organism


class MutableOrderList(UserList.UserList, organism.Mutable):
  """MutableOrderList implements a list; the order of its items can be
changed (by mutation, ...) but the number and the value of these items
don't change.

Instance of MutableOrderList are designed to be used as gene's values on a
chromosom, e.g. :

>>> chromosom = organism.Chromosom(order = mutablelist.MutableOrderList([1, 2, 3]))

Usefull for TSP (see ./demo/demo_saleman_2.py) !
"""
  def __init__(self, items = None):
    UserList.UserList .__init__(self, items)
    
  def mutate(self, chromosom):
    mutated = self
    
    for i in range(len(self)):
      if random.random() < chromosom.__mutation__:
        if mutated is self: mutated = self.__class__(self.data[:])
        mutated[i - 1], mutated[i] = mutated[i], mutated[i - 1]
    
    return mutated
