
import operator, random
import organism, copy

class NeuralFunc:
  def __init__(self, rootneuron):
    self.rootneuron = rootneuron
    
  def __call__(self, **args): return self.rootneuron(**args)
  
  def __repr__(self): return `self.rootneuron`
  

class Neuron(organism.Chromosom):
  DEFAULT_GENES = {
    "__mutation__"  :  0.5,
    "__mutampl__"   :  1.0,
    "__mutsign__"   :  0.0,
    "__loss__"      :  0.4,
    "__crossover__" :  0.0,
    "__split__"     :  0.05,
    }
  
  def __init__(self, **genes):
    self.synapses = []
    organism.Chromosom.__init__(self, **genes)
    self.neuralfunc = NeuralFunc(self)
    
  def value(self, **args):
    value = reduce(operator.add, [synaps(**args) for synaps in self.synapses], 0.0)
    #value = reduce(operator.add, map(Synaps.value, self.synapses), 0.0)
    if   value < -1.0: value = -1.0
    elif value >  1.0: value =  1.0
    return value
  __call__ = value
  
  def useless(self): return len(self.synapses) == 0
  
  def crossover(self, other): # No cross-over here !
    pass
  
  def mutate(self, chromosom = None):
    newdict = self.mutate_object(self.__dict__)
    
    #if newdict: mutated = copy.deepcopy(self)
    if newdict: mutated = self.__class__(**newdict)
    else:       mutated = self
    
    # Check for synaps loss (synapses with Constant or Input source cannot be lost in the "root" neuron. Else we may definitively loose some variable !) :
    synapses = filter(lambda synaps: ((chromosom is None) and not(isinstance(synaps, Neuron))) or random.random() > self.__loss__, mutated.synapses)
    if synapses != mutated.synapses:
      if mutated is self: mutated = self.__class__(**newdict)
      mutated.synapses = synapses
    
    # Check for neuron split :
    if random.random() < self.__split__:
      if mutated is self: mutated = self.__class__(**newdict)
      nn = Neuron()
      for synaps in self.synapses:
        nn.synapses.append(Synaps(synaps.source, 10.0 * (random.random() - 0.5)))
      mutated.synapses.append(Synaps(nn, 1.0))
      
    return mutated
  
  
  def __repr__(self): return "(%s)" % " + ".join(map(repr, self.synapses))

class Input:
  def __init__(self, name):
    self.name = name
    
  def __copy__    (self, memo): return self
  def __deepcopy__(self, memo): return self
    
  def value(self, **args): return args[self.name]
  __call__ = value
  
  def useless(self): return 0
  
  def __repr__(self): return self.name
  
  
class Constant:
  def __init__(self, value = 0.0):
    self._value = value
    
  def __copy__    (self, memo): return self
  def __deepcopy__(self, memo): return self
  
  def value(self, **args): return self._value
  __call__ = value
  
  def setvalue(self, value): self._value = value
  
  def useless(self): return 0
  
  def __repr__(self): return str(self._value)
  

class Synaps(organism.Mutable):
  def __init__(self, source, factor = 0.0):
    self.source = source
    self.factor = factor
    
  def value(self, **args):
    value = self.source(**args) * self.factor
    #if   value < -1.0: value = -1.0
    #elif value >  1.0: value =  1.0
    return value
  __call__ = value
  
  def mutate(self, chromosom):
    source = chromosom.mutate_object(self.source)
    factor = chromosom.mutate_object(self.factor)
    
    if not (source is self.source and factor is self.factor): return Synaps(source, factor)
    return self
    
  def __repr__(self):
    return "%s * %s" % (self.factor, `self.source`)
  
